%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI dialog for displaying graphs of            %
%          cointegration relations                               %
%                                                                %
%          Function: bayes_cointgraphdlg                         %
%                                                                %
%          Input:                                                %
%                       selector: case switch function           %
%                       c_mod: structure with model info         %
%                       posterior_data: structure with parameter %
%                          draws from the posterior distribution %
%                       joint_draws: 1 if using joint draws, 0   %
%                          otherwise                             %
%                                                                %
%          Output:                                               %
%                       h: handle                                %
%                                                                %
%          This version:        May 4, 2006                      %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = bayes_cointgraphdlg(selector,c_mod,posterior_data,joint_draws)

if strcmp(selector,'init')==1;
   tm = c_mod;
   thisexit = 0;
   listnumbers = '1';
   i = 2;
   while i<=tm.r;
      listnumbers = strcat(listnumbers,'|',num2str(i));
      i = i+1;
   end;
   curr_ini = find_inisettings;
   if isnumeric(curr_ini.external)==0;
      curr_ini.external = 0;
   end;
   %
   % setting up the actual sample period
   %
   [tm.dataindex,tm.samplebeg,tm.sampleend,add] = make_dataindex(tm.freq,tm.samplebeg,tm.sampleend);
   tm.dataindexfix = [tm.samplebeg-3*add;tm.samplebeg-2*add;tm.samplebeg-add;tm.dataindex];
   checkyear = floor(tm.samplebeg);
   checkperiod = tm.samplebeg-checkyear;
   tm.subbeginyear = num2str(checkyear);
   if strcmp(tm.freq,'q')==1;
      checkperiod = 4*(checkperiod+(1/8));
      tm.subbeginperiod = num2str(round(checkperiod));
   elseif strcmp(tm.freq,'m')==1;
      checkperiod = 12*(checkperiod+(1/24));
      tm.subbeginperiod = num2str(round(checkperiod));
   end;
   %
   % prepare data series
   %
   mywait = waitdlg(strvcat(' ','Preparing data series for graphics','Please wait!'),'information','Cointegration Relations',350,124);
   waitf = get(mywait,'UserData');
   %
   % Step 1: The integration matrix N
   %
   if isempty(tm.Z2)==0;
      N = eye(tm.T)-tm.Z2'*inverse(tm.Z2*tm.Z2')*tm.Z2;
   else;
      N = eye(tm.T);
   end;
   %
   % Step 2: sort the values for each t
   %
   Z = tm.X0;
   R = tm.X0*N;
   tm.z_mean = zeros(tm.r,tm.T);
   tm.z_median = zeros(tm.r,tm.T);
   tm.z_mode = zeros(tm.r,tm.T);
   tm.z_upper = zeros(tm.r,tm.T);
   tm.z_lower = zeros(tm.r,tm.T);
   tm.r_mean = zeros(tm.r,tm.T);
   tm.r_median = zeros(tm.r,tm.T);
   tm.r_mode = zeros(tm.r,tm.T);
   tm.r_upper = zeros(tm.r,tm.T);
   tm.r_lower = zeros(tm.r,tm.T);
   %
   up = (1+(tm.cbs/100))/2;
   up = floor(up*tm.save_draws);
   lp = (tm.save_draws)-up+1;
   mid_point = tm.save_draws/2;
   %
   for t=1:tm.T;
      Zt = Z(:,t);
      Rt = R(:,t);
      z_values = zeros(tm.save_draws,tm.r);
      r_values = zeros(tm.save_draws,tm.r);
      for i=1:tm.save_draws;
         if joint_draws==1;
            bZt = posterior_data(i).beta'*Z(:,t);
            bRt = posterior_data(i).beta'*R(:,t);
         else;
            bZt = posterior_data(i).beta_n'*Z(:,t);
            bRt = posterior_data(i).beta_n'*R(:,t);
         end;
         z_values(i,:) = bZt';
         r_values(i,:) = bRt';
      end;
      tm.z_mean(:,t) = (1/tm.save_draws)*sum(z_values',2);
      tm.r_mean(:,t) = (1/tm.save_draws)*sum(r_values',2);
      %
      % sort the values
      %
      for i=1:tm.r;
         z_values(:,i) = sort(z_values(:,i));
         r_values(:,i) = sort(r_values(:,i));
         %
         % compute the mode based on the Epanechnikov kernel density method
         %
         z_min = min(z_values(:,i));
         z_max = max(z_values(:,i));
         z_range = z_max-z_min;
         az = z_min-z_range/4;
         bz = z_max+z_range/4;
         h_parm = hns(z_values(:,i),'kepan');
         [x_v,y_v] = kdfft1(z_values(:,i),'kepan',512,h_parm,0,[az,bz]);
         [max_y,pos_max] = max(y_v);
         tm.z_mode(i,t) = x_v(1,pos_max);
         %
         r_min = min(r_values(:,i));
         r_max = max(r_values(:,i));
         r_range = r_max-r_min;
         ar = r_min-r_range/4;
         br = r_max+r_range/4;
         h_parm = hns(r_values(:,i),'kepan');
         [x_v,y_v] = kdfft1(r_values(:,i),'kepan',512,h_parm,0,[ar,br]);
         [max_y,pos_max] = max(y_v);
         tm.r_mode(i,t) = x_v(1,pos_max);
         %
         % compute the median
         %
         tm.z_median(i,t) = median(z_values(:,i));
         tm.r_median(i,t) = median(r_values(:,i));
      end;
      %
      if t==tm.T;
         tm.z_values = z_values;
      end;
      %
      tm.z_upper(:,t) = z_values(up,:)';
      tm.z_lower(:,t) = z_values(lp,:)';
      tm.r_upper(:,t) = r_values(up,:)';
      tm.r_lower(:,t) = r_values(lp,:)';
   end;
   set(waitf.text,'UserData','done');
   delete(mywait);
   drawnow;
   if joint_draws==1;
      fig_name = 'Joint';
   else;
      fig_name = 'Partial';
   end;
   tm.fig_name = fig_name;
else;
   thisexit = 1;
   cointgraphgui = get(0,'CurrentFigure');
   graph = get(cointgraphgui,'UserData');
   curr_ini = get(graph.popup,'UserData');
   tm = curr_ini.models;
end;

%
% Set screen size directly instead of using
% get(0,'ScreenSize')!
%

realscrsz = get(0,'ScreenSize');

switch selector;
   case 'init';
      cointgraphgui = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','bayes_cointgraphdlg.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(realscrsz(3)-341)/2 (realscrsz(4)-140)/2 341 165], ...
         'Tag','FigParent', ...
         'Resize','off', ...
         'Visible','off', ...
         'Name',['Cointegration Relations - ' fig_name ], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','bayes_cointgraphdlg done', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(cointgraphgui,'DockControl','off');
      end;

      %
      % Outer axis
      %

      axesbox([2 2 339 155],'Graphics',45,[0.5 1]);

      imageaxes = axes('Units','pixels', ...
         'Box','on', ...
         'TickLength',[0 0], ...
         'CameraUpVector',[0 1 0], ...
         'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 105 39 39], ...
         'XTickLabelMode','manual', ...
         'YTickLabelMode','manual', ...
         'Tag','AxisLogo', ...
         'XColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'YColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'ZColor',get(0,'defaultuicontrolbackgroundcolor'));

      mycons = getstoredimages;
      if curr_ini.use_toolbar_bgcolor==1;
         bgcolor = (1/255)*(round(255*get(0,'defaultuicontrolbackgroundcolor'))-[8 4 8]);
         bgcolor(1) = max(0,bgcolor(1));
         bgcolor(2) = max(0,bgcolor(2));
         bgcolor(3) = max(0,bgcolor(3));
      else;
         bgcolor = get(0,'defaultuicontrolbackgroundcolor');
      end;
      %
      image('CData',mirrorimage(maketrans(mycons.cointbig_image,[160 160 164],bgcolor)),'Clipping','off','Parent',imageaxes);

      graph.icon = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 75 200 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String',['Estimation Sample:    ' tm.subbeginyear ':' tm.subbeginperiod ' - ' tm.subendyear ':' tm.subendperiod], ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');

      %
      % Select variable
      %

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 40 120 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','Select Relation Number:', ...
         'HorizontalAlignment','left', ...
         'Tag','VariableText');

      graph.popup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[144 45 60 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String',listnumbers, ...
         'Tag','GraphListbox',...
         'Value',1);

      graph.center_series = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 16 170 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','checkbox', ...
         'Value',0, ...
         'Enable','on', ...
         'Visible','on', ...
         'HorizontalAlignment','left', ...
         'String','Center time series', ...
         'Tag','CheckCenterSeries');


      if tm.scale_endo~=1;
         graph.scale_vars = uicontrol('Units','pixels', ...
            'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
            'Position',[144 16 100 20], ...
            'FontSize',curr_ini.gui_fontsize, ...
            'FontName',curr_ini.gui_fontname, ...
            'FontWeight',curr_ini.gui_fontweight, ...
            'FontAngle',curr_ini.gui_fontangle, ...
            'Style','checkbox', ...
            'Value',0, ...
            'Enable','on', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Rescale', ...
            'Tag','CheckRescaleVariables');
      end;

      %
      % Create graph button
      %

      graph.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[236 55 90 20], ...
         'String','Display', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','bayes_cointgraphdlg showgraph', ...
         'UserData','working', ...
         'Enable','on', ...
         'Tag','Done');

      %
      %  Done button
      %

      graph.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[236 20 90 20], ...
         'String','Done', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','bayes_cointgraphdlg done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');

      set(cointgraphgui,'UserData',graph,...
         'HandleVisibility','callback');
   case 'showgraph'
      series = get(graph.popup,'Value');
      newvalue = series+1;
      if newvalue>tm.r;
         set(graph.popup,'Value',1);
      else;
         set(graph.popup,'Value',newvalue);
      end;
      if curr_ini.colors==1;
         colorstr1 = [0 0 0];
         colorstr2 = (1/255)*[255 255 177];
         colorstr3 = [1 0 0];
         colorstr4 = [0 1 0];
         colorstr5 = [0 0 1];
      else;
         colorstr1 = [0 0 0];
         colorstr2 = [0.6 0.6 0.6];
         colorstr3 = [0.45 0.45 0.45];
         colorstr4 = [0.3 0.3 0.3];
         colorstr5 = [0.15 0.15 0.15];
      end;
      %
      realscrsz = get(0,'ScreenSize');
      pos = [(realscrsz(3)-612)/2 (realscrsz(4)-484)/2];
      %
      % rescale based on the inverse of the scale factor
      %
      if isfield(graph,'scale_vars')==1;
         if get(graph.scale_vars,'Value')==1;
            scale_vars = 1/tm.scale_endo;
         else;
            scale_vars = 1;
         end;
      else;
         scale_vars = 1;
      end;
      z_upper = scale_vars*tm.z_upper(series,:);
      z_lower = scale_vars*tm.z_lower(series,:);
      z_mean = scale_vars*tm.z_mean(series,:);
      z_median = scale_vars*tm.z_median(series,:);
      z_mode = scale_vars*tm.z_mode(series,:);
      if get(graph.center_series,'Value')==1;
         mean_z = (1/tm.T)*sum(z_mean);
         z_upper = z_upper-mean_z;
         z_lower = z_lower-mean_z;
         z_mean = z_mean-mean_z;
         z_median = z_median-mean_z;
         z_mode = z_mode-mean_z;
      end;
      r_upper = scale_vars*tm.r_upper(series,:);
      r_lower = scale_vars*tm.r_lower(series,:);
      r_mean = scale_vars*tm.r_mean(series,:);
      r_median = scale_vars*tm.r_median(series,:);
      r_mode = scale_vars*tm.r_mode(series,:);
      %
      % Display results
      %
      fig_handle_1 = figure('Units','pixels', ...
         'Position',[pos 612 484], ...
         'Menubar','none', ...
         'Toolbar','none', ...
         'NumberTitle','off', ...
         'Tag',['beta-series-' num2str(series,'%0.0f') '-'], ...
         'Visible','off', ...
         'Name',['Time Series Plot of Cointegration Relation no. ' num2str(series,'%0.0f')]);
      %
      % default figure properties
      %
      setfigureproperties(fig_handle_1);
      %
      % Using a patch to display the confidence region
      %
      my_p = patch([tm.dataindex' reverse(tm.dataindex')],[z_upper reverse(z_lower)],colorstr2);
      set(my_p,'FaceColor',colorstr2,'EdgeColor',colorstr2,'Tag','confpatch','LineWidth',1);
      hold('on');
      %
      % mean values
      %
      line_mean = plot(tm.dataindex',z_mean,'-','LineWidth',1,'Color',colorstr3,'Tag',['mean-beta-series-' num2str(series,'%0.0f')]);
      hold('on');
      %
      % median values
      %
      line_median = plot(tm.dataindex',z_median,':','LineWidth',1,'Color',colorstr4,'Tag',['median-beta-series-' num2str(series,'%0.0f')]);
      hold('on');
      %
      % mode values
      %
      line_mode = plot(tm.dataindex',z_mode,'--','LineWidth',1,'Color',colorstr5,'Tag',['mode-beta-series-' num2str(series,'%0.0f')]);
      hold('on');
      %
      set(gca,'XLim',[min(tm.dataindex) max(tm.dataindex)],'Layer','top');
      %
      % plot a zero line if needed
      %
      y_lim = get(gca,'YLim');
      if (y_lim(1)<0)&(y_lim(2)>0);
         plot([min(tm.dataindex) max(tm.dataindex)],[0 0],'-','LineWidth',1,'Color',[0 0 0]);
         hold('on');
      end;
      title_str = ['Cointegration relation no. ' num2str(series,'%0.0f') ' with ' num2str(tm.c,'%0.0f') '% confidence region'];
      setaxesfonts('Title',title_str);
      %
      % create a legend
      %
      myleg = legend_fcn([line_mean line_median line_mode my_p],'Mean','Median','Mode','Confidence',2);
      setaxesfonts(myleg);
      set(gca,'UserData',myleg);
      hold('off');
      toolbar_fcn(gca);
      %
      pos = pos+(22*[1 -1]);
      fig_handle_2 = figure('Units','pixels', ...
         'Position',[pos 612 484], ...
         'Menubar','none', ...
         'Toolbar','none', ...
         'NumberTitle','off', ...
         'Tag',['beta-series-corr-' num2str(series,'%0.0f') '-'], ...
         'Visible','off', ...
         'Name',['Time Series Plot of Cointegration Relation no. ' num2str(series,'%0.0f') ' - Corrected']);
      %
      % default figure properties
      %
      setfigureproperties(fig_handle_2);
      %
      % Using a patch to display the confidence region
      %
      my_p = patch([tm.dataindex' reverse(tm.dataindex')],[r_upper reverse(r_lower)],colorstr2);
      set(my_p,'FaceColor',colorstr2,'EdgeColor',colorstr2,'Tag','confpatch','LineWidth',1);
      hold('on');
      %
      % mean values
      %
      line_mean = plot(tm.dataindex',r_mean,'-','LineWidth',1,'Color',colorstr3,'Tag',['mean-beta-corr-series-' num2str(series,'%0.0f')]);
      hold('on');
      %
      % median values
      %
      line_median = plot(tm.dataindex',r_median,':','LineWidth',1,'Color',colorstr4,'Tag',['median-beta-corr-series-' num2str(series,'%0.0f')]);
      hold('on');
      %
      % mode values
      %
      line_mode = plot(tm.dataindex',r_mode,'--','LineWidth',1,'Color',colorstr5,'Tag',['mode-beta-corr-series-' num2str(series,'%0.0f')]);
      hold('on');
      %
      set(gca,'XLim',[min(tm.dataindex) max(tm.dataindex)],'Layer','top');
      %
      % plot a zero line if needed
      %
      y_lim = get(gca,'YLim');
      if (y_lim(1)<0)&(y_lim(2)>0);
         plot([min(tm.dataindex) max(tm.dataindex)],[0 0],'-','LineWidth',1,'Color',[0 0 0]);
         hold('on');
      end;
      title_str = ['Corrected cointegration relation no. ' num2str(series,'%0.0f') ' with ' num2str(tm.c,'%0.0f') '% confidence region'];
      setaxesfonts('Title',title_str);
      %
      % create a legend
      %
      myleg = legend_fcn([line_mean line_median line_mode my_p],'Mean','Median','Mode','Confidence',2);
      setaxesfonts(myleg);
      set(gca,'UserData',myleg);
      hold('off');
      toolbar_fcn(gca);
      %
      % plotting the cross-section distribution for the last observation
      %
      z_values = scale_vars*tm.z_values(:,series);
      if get(graph.center_series,'Value')==1;
         z_values = z_values-mean_z;
      end;
      if curr_ini.colors==1;
         col.vec_1 = [0 0 1];
         col.vec_2 = [1 0 0];
         col.vec_3 = [0 0.75 0];
         col.vec_4 = [1 0.5 0];
         col.vec_5 = [0 0.75 0.75];
         col.vec_6 = [0 0 0];
         col.vec_7 = (1/255)*[255 28 28];
         col.vec_8 = [0 1 0];
         col.vec_9 = (1/255)*[255 255 177];
      else;
         col.vec_1 = [0 0 0];
         col.vec_2 = [0.7 0.7 0.7];
         col.vec_3 = [0.6 0.6 0.6];
         col.vec_4 = [0.5 0.5 0.5];
         col.vec_5 = [0.4 0.4 0.4];
         col.vec_6 = [0.3 0.3 0.3];
         col.vec_7 = [0.8 0.8 0.8];
         col.vec_8 = [0.2 0.2 0.2];
         col.vec_9 = col.vec_3;
      end;
      %
      pos = pos+(22*[1 -1]);
      tm.realscrsz = realscrsz;
      tm.graph_type = 'function';
      [pos,fig_handle_3] = bayes_parm_dist(pos,['coint-' num2str(series,'%0.0f') '-' tm.subendyear '.' tm.subendperiod],['Cointegration Relation No. ' num2str(series,'%0.0f') ' - ' tm.fig_name],['Cointegration relation no. ' num2str(series,'%0.0f') ' for ' tm.subendyear ':' tm.subendperiod ' with ' num2str(tm.c,'%0.0f') '% confidence region'],tm,curr_ini,z_values,['Coint. Rel. ' num2str(series,'%0.0f')],col);
      %


      if curr_ini.external==0;
         set([fig_handle_1 fig_handle_2 fig_handle_3],'Visible','on');
         drawnow;
      else;
         file1 = [tm.outputdir tm.modelfile '-' get(fig_handle_1,'Tag') num2str(tm.n) '.' num2str(tm.r) '-' num2str(tm.save_draws,'%0.0f')];
         set(0,'CurrentFigure',fig_handle_1);
         stat1 = autosavefigure(fig_handle_1,file1);
         %
         file2 = [tm.outputdir tm.modelfile '-' get(fig_handle_2,'Tag') num2str(tm.n) '.' num2str(tm.r) '-' num2str(tm.save_draws,'%0.0f')];
         set(0,'CurrentFigure',fig_handle_2);
         stat2 = autosavefigure(fig_handle_2,file2);
         %
         file3 = [tm.outputdir tm.modelfile '-' get(fig_handle_3,'Tag') num2str(tm.n) '.' num2str(tm.r) '-' num2str(tm.save_draws,'%0.0f')];
         set(0,'CurrentFigure',fig_handle_3);
         stat3 = autosavefigure(fig_handle_3,file3);
         if isempty(stat1)==0;
            if stat1==0;
               autoshowfigure(file1);
            end;
         end;
         if isempty(stat2)==0;
            if stat2==0;
               autoshowfigure(file2);
            end;
         end;
         if isempty(stat3)==0;
            if stat3==0;
               autoshowfigure(file3);
            end;
         end;
      end;
   case 'done'
      set(graph.done,'UserData','done');
end;

if nargout==1;
   h = cointgraphgui;
end;

if strcmp(selector,'done')==0;
   curr_ini.models = tm;
   set(graph.popup,'UserData',curr_ini);
   set(cointgraphgui,'UserData',graph);
end;

if thisexit==0;
   if (iswinnt==1)&(iscompiled==1)&(curr_ini.loadguiicon==1);
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(cointgraphgui,'Name') ' ' fix_filepath([pwd '\icons\coint.ico'])];
      dos(prog);
      if (strcmp(curr_ini.pstoolsdir,'')==0)&(appexist([curr_ini.pstoolsdir 'pslist.exe'])==1);
         %
         % now we need to determine the pid
         %
         pid = getpid('seticon');
         set(graph.icon,'UserData',pid);
      end;
   end;
   %
   set(cointgraphgui,'Visible','on');
   drawnow;
   if (iswinnt==1)&(iscompiled==1)&(matlabnumber<7);
      drawnow;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(cointgraphgui,'Name') ' 1'];
      dos(prog);
      drawnow;
   elseif matlabnumber>=7;
      uicontrol(graph.done);
   end;
   waitfor(graph.done,'UserData','done');
end;

%
% end of bayes_cointgraphdlg.m
%
