%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          Computes multivariate Omnibus test for normality      %
%          as suggested by Doornik-Hansen (and based on          %
%          Shenton-Bowman)                                       %
%                                                                %
%          Function: omnibus_fcn                                 %
%                                                                %
%          Input:                                                %
%                       epsilon: nxT matrix with residuals       %
%                       t: degrees of freedom from original      %
%                           estimates                            %
%                                                                %
%          Output:                                               %
%          	            normalityvec: vector with results        %
%                       df: string with degrees of freedom       %
%                                                                %
%          This version:        June 1, 2004                     %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2004          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [normalityvec,df] = omnibus_fcn(epsilon,t)

[n T] = size(epsilon);

curr_ini = find_inisettings;

%
% Normalizing residuals
%

Omega = (1/T)*epsilon*epsilon';
V = inverse(sqrt(diag(diag(Omega))));
C = V*Omega*V;
[H,lambda] = eig(C);
w = H*inverse(sqrt(lambda))*H'*V*epsilon;

%
% An alternative normalization
%
% P = chol(Omega)';
% w = inverse(P)*epsilon;
%

%
% Estimating 3rd and 4th moments
%

b1 = w.*w.*w;
b2 = b1.*w;
b1 = (1/T)*sum(b1')';
b2 = (1/T)*sum(b2')';

z1 = [];
z2 = [];
beta = ((3*((T*T)+(27*T)-70)*(T+1)*(T+3))/((T-2)*(T+5)*(T+7)*(T+9)));
omega = sqrt((2*(beta-1)))-1;
delta = 1/(sqrt(log(sqrt(omega))));
psi = sqrt((((omega-1)*(T+1)*(T+3))/(12*(T-2))));
d = (T-3)*(T+1)*((T*T)+(15*T)-4);
a = ((T-2)*(T+5)*(T+7)*((T*T)+(27*T)-70))/(6*d);
c = ((T-7)*(T+5)*(T+7)*((T*T)+(2*T)-5))/(6*d);
k = ((T+5)*(T+7)*((T*T*T)+(37*T*T)+(11*T)-313))/(12*d);

for i=1:n;
  y = b1(i,1)*psi;
  z1i = delta*log((y+sqrt(((y*y)+1))));
  z1 = [z1 z1i];
  alpha = a+(b1(i,1)*b1(i,1)*c);
  xi = 2*k*(b2(i,1)-1-(b1(i,1)*b1(i,1)));
  z2i = sqrt(9*alpha)*((xi/(2*alpha))^(1/3)-1+(1/(9*alpha)));
  z2 = [z2 z2i];
end;

z1 = z1';
z2 = z2';

%
% Computing the omnibus test
%

Ep = (z1'*z1)+(z2'*z2);

if curr_ini.sscorrect==0;
   df = num2str(2*n);
   pval = chis_cdf(Ep,2*n);
   normalityvec = [Ep 1-pval];
else;
   df = ['(' num2str(2*n) ',' num2str(t-2) ')'];
   Ep = ((t-2)/(T*2*n))*Ep;
   pval = fdis_cdf(Ep,2*n,t-2);
   normalityvec = [Ep 1-pval];
end;

%
% end of omnibus_fcn.m
%
