%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for computing Wald or F-tests         %
%          of a variable being weakly exogenous with respect     %
%          to (alpha,beta)                                       %
%                                                                %
%          Function: weakexotest                                 %
%                                                                %
%          Input:                                                %
%          	            alpha: nxr matrix                        %
%                       beta: (n+q1+dcoint)xr matrix             %
%                       epsilon: nxT matrix                      %
%                       Z1: (n(p-1)+d+q0p+q1(p-1))xT matrix      %
%                       Z2: (n+q1+dcoint)xT matrix               %
%                       t: number of effective observations      %
%                           (T-#free parameters)                 %
%                       ind: 0-1 indicator to use restrictions   %
%                           on alpha under the null and the      %
%                           alternative (default=0)              %
%                       moddir: path to G-modelfile.rest         %
%                           files (default=empty string)         %
%                       modfile: name of model (default is       %
%                           empty string)                        %
%                       Sigma_pi: covariance matrix for Psi      %
%                           and alpha                            %
%                       c_ind: indicator if C restrictions       %
%                           should be included                   %
%                                                                %
%          Output:      results: matrix with results             %
%                       df: matrix with degrees of freedom       %
%                                                                %
%          This version:        January 20, 2005                 %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2005          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [results,df] = weakexotest(alpha,beta,epsilon,Z1,Z2,t,ind,moddir,modfile,Sigma_pi,c_ind,Psi,p,pe,d,q1,q0)

if nargin<7;
   ind = 0;
   moddir = '';
   modfile = '';
   Sigma_pi = [];
end;

if nargin<11;
   c_ind = 0;
   Psi = [];
   p = 0;
   pe = 0;
   d = 0;
   q1 = 0;
   q0 = 0;
end;

%
% Preliminaries
%

[n r] = size(alpha);
T = size(epsilon,2);
Z3 = [Z2;(beta'*Z1)];
n1 = size(Z3,1);
Omega = (1/T)*epsilon*epsilon';
if ind==0;
   if c_ind==0;
      Sigma_pi = kron(inverse(Z3*Z3'),Omega);
   end;
   G = [];
else;
   %
   % Get the restrictions file
   %
   G = [];
   file = strcat(moddir,'G-',int2str(n),'x',int2str(r),'-',modfile,'.rest');
   if fileexist(file)==1;
      G = loaddata(file);
   end;
end;
n2 = size(Sigma_pi,1);
VarAlpha = Sigma_pi(n2-(n*r)+1:n2,n2-(n*r)+1:n2);
valpha = vec(alpha);

if c_ind==1;
   F = [];
   file = strcat(moddir,'C-',int2str(n),'x',int2str(r),'-',modfile,'.rest');
   if fileexist(file)==1;
      F = loaddata(file);
   end;
else;
   F = [];
end;

curr_ini = find_inisettings;

%
% Starting up the loop over each variable
%

if curr_ini.sscorrect==1;
   df = zeros(n,2);
else;
   df = zeros(n,1);
end;

results = zeros(n,2);

for i=1:n;
   R = zeros(r,n*r);
   for j=1:r;
      R(j,i+n*(j-1)) = 1;
   end;
   nr = size(R,1);
   if ind==0;
      if isempty(F)==0;
         [Rmat_C,rvec_C] = rmatrix(F,n,n);
         if r<n;
            C = cmat(Psi,alpha,beta,epsilon,Z1,Z2,p,d,0);
         else;
            C = cmat_stat(Psi,alpha,beta,epsilon,Z1,Z2,p,d,0,[]);
         end;
         xi = xi_mat(C,Psi,alpha,p,pe,r,d,q1,q0);
         Rmat_C = Rmat_C*kron(xi',C);
         if rank(Rmat_C)<size(Rmat_C,1);
            [red_rest,use_rest] = getdelvars(Rmat_C',[]);
            Rmat_C = Rmat_C(use_rest,:);
            rvec_C = rvec_C(use_rest,1);
         end;
         dfc = size(Rmat_C,1);
         if size(Rmat_C,2)>size(R,2);
            R_add = zeros(size(R,1),size(Rmat_C,2)-size(R,2));
         else;
            R_add = [];
         end;
         [delrest,keeprest] = getdelvars([Rmat_C' [R_add R]'],[]);
         if size(keeprest,1)==dfc;
            nr = 0;
            W = 0;
         elseif size(keeprest,1)==dfc+nr;
            Hcheck = null([Rmat_C;[R_add R]]);
            thisR = [Rmat_C;[R_add R]];
            thisr = [rvec_C;zeros(size(R,1),1)];
            h_check = (thisR'/(thisR*thisR'))*thisr;
            checkalpha = Hcheck*randn(size(Hcheck,2),1)+h_check;
            checkalpha = vecinv(checkalpha,n);
            checkalpha = checkalpha(:,size(checkalpha,2)-r+1:size(checkalpha,2));
            if rank(checkalpha)==r;
               W = (R*valpha)'*inverse(R*VarAlpha*R')*(R*valpha);
            else;
               nr = 0;
               W = 0;
            end;
         else;
            keeprest = keeprest(dfc+1:size(keeprest,1),1);
            keeprest = keeprest-dfc;
            R = R(keeprest,:);
            if isempty(R_add)==0;
               R_add = R_add(keeprest,:);
            end;
            Hcheck = null([Rmat_C;[R_add R]]);
            thisR = [Rmat_C;[R_add R]];
            thisr = [rvec_C;zeros(size(R,1),1)];
            h_check = (thisR'/(thisR*thisR'))*thisr;
            checkalpha = Hcheck*randn(size(Hcheck,2),1)+h_check;
            checkalpha = vecinv(checkalpha,n);
            checkalpha = checkalpha(:,size(checkalpha,2)-r+1:size(checkalpha,2));
            if rank(checkalpha)==r;
               nr = size(R,1);
               W = (R*valpha)'*inverse(R*VarAlpha*R')*(R*valpha);
            else;
               nr = 0;
               W = 0;
            end; 
         end;
      else;
         W = (R*valpha)'*inverse(R*VarAlpha*R')*(R*valpha);
      end;
   else;
      if isempty(F)==0;
         [Rmat_C,rvec_C] = rmatrix(F,n,n);
         if r<n;
            C = cmat(Psi,alpha,beta,epsilon,Z1,Z2,p,d,0);
         else;
            C = cmat_stat(Psi,alpha,beta,epsilon,Z1,Z2,p,d,0,[]);
         end;
         xi = xi_mat(C,Psi,alpha,p,pe,r,d,q1,q0);
         Rmat_C = Rmat_C*kron(xi',C);
         [Rmat,rvec] = rmatrix(G,n,r);
         if size(Rmat_C,2)>size(Rmat,2);
            Rmat = [zeros(size(Rmat,1),size(Rmat_C,2)-size(Rmat,2)) Rmat];
         end;
         Rmat_C = [Rmat;Rmat_C];
         rvec_C = [rvec;rvec_C];
         if rank(Rmat_C)<size(Rmat_C,1);
            [red_rest,use_rest] = getdelvars(Rmat_C',[]);
            Rmat_C = Rmat_C(use_rest,:);
            rvec_C = rvec_C(use_rest,1);
         end;
         dfc = size(Rmat_C,1);
         if size(Rmat_C,2)>size(R,2);
            R_add = zeros(size(R,1),size(Rmat_C,2)-size(R,2));
         else;
            R_add = [];
         end;
         [delrest,keeprest] = getdelvars([Rmat_C' [R_add R]'],[]);
         if size(keeprest,1)==dfc;
            nr = 0;
            W = 0;
         elseif size(keeprest,1)==dfc+nr;
            Hcheck = null([Rmat_C;[R_add R]]);
            thisR = [Rmat_C;[R_add R]];
            thisr = [rvec_C;zeros(size(R,1),1)];
            h_check = (thisR'/(thisR*thisR'))*thisr;
            checkalpha = Hcheck*randn(size(Hcheck,2),1)+h_check;
            checkalpha = vecinv(checkalpha,n);
            checkalpha = checkalpha(:,size(checkalpha,2)-r+1:size(checkalpha,2));
            if rank(checkalpha)==r;
               W = (R*valpha)'*inverse(R*VarAlpha*R')*(R*valpha);
            else;
               nr = 0;
               W = 0;
            end;
         else;
            keeprest = keeprest(dfc+1:size(keeprest,1),1);
            keeprest = keeprest-dfc;
            R = R(keeprest,:);
            if isempty(R_add)==0;
               R_add = R_add(keeprest,:);
            end;
            Hcheck = null([Rmat_C;[R_add R]]);
            thisR = [Rmat_C;[R_add R]];
            thisr = [rvec_C;zeros(size(R,1),1)];
            h_check = (thisR'/(thisR*thisR'))*thisr;
            checkalpha = Hcheck*randn(size(Hcheck,2),1)+h_check;
            checkalpha = vecinv(checkalpha,n);
            checkalpha = checkalpha(:,size(checkalpha,2)-r+1:size(checkalpha,2));
            if rank(checkalpha)==r;
               nr = size(R,1);
               W = (R*valpha)'*inverse(R*VarAlpha*R')*(R*valpha);
            else;
               nr = 0;
               W = 0;
            end; 
         end;
      else;
         if isempty(G)==0;
            [Rmat,rvec] = rmatrix(G,n,r);
            dfa = size(Rmat,1);
            [delrest,keeprest] = getdelvars([Rmat' R'],[]);
            if size(keeprest,1)==dfa;
               nr = 0;
               W = 0;
            elseif size(keeprest,1)==dfa+nr;
               Hcheck = null([Rmat;R]);
               thisR = [Rmat;R];
               thisr = [rvec;zeros(size(R,1),1)];
               h_check = (thisR'/(thisR*thisR'))*thisr;
               checkalpha = Hcheck*randn(size(Hcheck,2),1)+h_check;
               checkalpha = vecinv(checkalpha,n);
               if rank(checkalpha)==r;
                  W = (R*valpha)'*inverse(R*VarAlpha*R')*(R*valpha);
               else;
                  nr = 0;
                  W = 0;
               end; 
            else;
               keeprest = keeprest(dfa+1:size(keeprest,1),1);
               keeprest = keeprest-dfa;
               R = R(keeprest,:);
               Hcheck = null([Rmat;R]);
               thisR = [Rmat;R];
               thisr = [rvec;zeros(size(R,1),1)];
               h_check = (thisR'/(thisR*thisR'))*thisr;
               checkalpha = Hcheck*randn(size(Hcheck,2),1)+h_check;
               checkalpha = vecinv(checkalpha,n);
               if rank(checkalpha)==r;
                  nr = size(R,1);
                  W = (R*valpha)'*inverse(R*VarAlpha*R')*(R*valpha);
               else;
                  nr = 0;
                  W = 0;
               end; 
            end;
         else;
            W = (R*valpha)'*inverse(R*VarAlpha*R')*(R*valpha);
         end;
      end;
   end;
   if nr>0;
      if curr_ini.sscorrect==1;
         results(i,1) = ((t-nr)/(T*nr))*W;
         df(i,:) = [nr t-nr];
         results(i,2) = 1-fdis_cdf(results(i,1),nr,t-nr);
      else;
         results(i,1) = W;
         df(i,1) = nr;
         results(i,2) = 1-chis_cdf(W,nr);
      end;
   end;
end;

%
% end of weakexotest.m
%
